/*
 * Created on 10.11.2004
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2446 $
 *  $Date: 2006-09-12 14:57:25 +0200 (Di, 12 Sep 2006) $
 *  $Id: LoadPirolCsvDatasetPlugIn.java 2446 2006-09-12 12:57:25Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.datasources.pirolCSV;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import com.vividsolutions.jts.util.Assert;
import com.vividsolutions.jump.coordsys.CoordinateSystemRegistry;
import com.vividsolutions.jump.feature.FeatureCollection;
import com.vividsolutions.jump.io.datasource.Connection;
import com.vividsolutions.jump.io.datasource.DataSourceQuery;
import com.vividsolutions.jump.task.TaskMonitor;
import com.vividsolutions.jump.util.CollectionUtil;
import com.vividsolutions.jump.util.StringUtil;
import com.vividsolutions.jump.workbench.WorkbenchContext;
import com.vividsolutions.jump.workbench.datasource.DataSourceQueryChooserDialog;
import com.vividsolutions.jump.workbench.datasource.DataSourceQueryChooserManager;
import com.vividsolutions.jump.workbench.plugin.EnableCheckFactory;
import com.vividsolutions.jump.workbench.plugin.MultiEnableCheck;
import com.vividsolutions.jump.workbench.plugin.PlugInContext;
import com.vividsolutions.jump.workbench.ui.GUIUtil;
import com.vividsolutions.jump.workbench.ui.WorkbenchFrame;
import com.vividsolutions.jump.workbench.ui.task.TaskMonitorDialog;

import de.fhOsnabrueck.jump.pirol.utilities.apiTools.CategoryTools;

/**
 * @author Ole Rahn
 *
 * FH Osnabrck - University of Applied Sciences Osnabrck
 * Project PIROL 2005
 * Daten- und Wissensmanagements
 */
public class LoadPirolCsvDatasetPlugIn extends com.vividsolutions.jump.workbench.datasource.LoadDatasetPlugIn {
    private static String LAST_FORMAT_KEY = LoadPirolCsvDatasetPlugIn.class.getName() +
    " - LAST FORMAT";
    
    private DataSourceQueryChooserDialog getDialog(PlugInContext context) {
        String KEY = LoadPirolCsvDatasetPlugIn.class.getName() + " - DIALOG";
        if (null == context.getWorkbenchContext().getWorkbench().getBlackboard()
                               .get(KEY)) {
            context.getWorkbenchContext().getWorkbench().getBlackboard().put(KEY,
                new DataSourceQueryChooserDialog(DataSourceQueryChooserManager.get(
                        context.getWorkbenchContext().getWorkbench()
                               .getBlackboard()).getLoadDataSourceQueryChoosers(),
                    context.getWorkbenchFrame(), LoadPirolCsvDatasetPlugIn.getThisName(), true));
        }

        return (DataSourceQueryChooserDialog) context.getWorkbenchContext()
                                                     .getWorkbench()
                                                     .getBlackboard().get(KEY);
    }
    
	public String getName() {
	    return LoadPirolCsvDatasetPlugIn.getThisName();
	}
	
	public static String getThisName(){
	    return "Load Pirol Csv Dataset";
	}

	public void initialize(final PlugInContext context) throws Exception {

		PirolCsvDatasetDataSource.setOwner(context.getWorkbenchFrame());
		DataSourceQueryChooserManager.get(context.getWorkbenchContext().getWorkbench().getBlackboard()).addLoadDataSourceQueryChooser(new LoadPirolCsvDatasetDataSourceQueryChooser(context.getWorkbenchContext()));
		
	}
	
    /**
     * This method is never executed, for some reason I don't understand the method of one of the parent classes can not be
     * overridden...
     *@param monitor
     *@param context
     *@throws Exception
     */
    public void run(TaskMonitor monitor, PlugInContext context)
           throws Exception {
       
		System.out.println("LoadPirolCsvDatasetPlugIn.run()");
		LoadPirolCsvDatasetDataSourceQueryChooser chooser = new LoadPirolCsvDatasetDataSourceQueryChooser(context.getWorkbenchContext());
		
		boolean exceptionsEncountered = false;
		
		
		for (Iterator i = chooser.getDataSourceQueries().iterator(); i.hasNext();) {
			DataSourceQuery dataSourceQuery = (DataSourceQuery) i.next();
			ArrayList exceptions = new ArrayList();
			Assert.isTrue(dataSourceQuery.getDataSource().isReadable());
			
			((TaskMonitorDialog)monitor).setVisible(false);

			monitor.report("Loading " + dataSourceQuery.toString() + " (Pirol-CSV) ...");
			monitor.allowCancellationRequests();
			((TaskMonitorDialog)monitor).setVisible(false);
			
			Connection connection = dataSourceQuery.getDataSource()
					.getConnection();
			try {
				FeatureCollection dataset = dataSourceQuery
						.getDataSource()
						.installCoordinateSystem(
								connection.executeQuery(dataSourceQuery
										.getQuery(), exceptions, monitor),
								CoordinateSystemRegistry.instance(context
										.getWorkbenchContext().getBlackboard()));
				if (dataset != null) {
					context.getLayerManager().addLayer(CategoryTools.getActiveCategory(context),
							dataSourceQuery.toString(), dataset)
							.setDataSourceQuery(dataSourceQuery)
							.setFeatureCollectionModified(false);
				}
			} finally {
				connection.close();
			}
			if (!exceptions.isEmpty()) {
				if (!exceptionsEncountered) {
					context.getOutputFrame().createNewDocument();
					exceptionsEncountered = true;
				}
				reportExceptions(exceptions, dataSourceQuery, context);
			}
		}
		if (exceptionsEncountered) {
			context
					.getWorkbenchFrame()
					.warnUser(
							"Problems were encountered. See Output Window for details.");
		}
	}

    public static MultiEnableCheck createEnableCheck(
			final WorkbenchContext workbenchContext) {
		EnableCheckFactory checkFactory = new EnableCheckFactory(
				workbenchContext);

		return new MultiEnableCheck().add(checkFactory
				.createWindowWithLayerManagerMustBeActiveCheck());
	}
	
	private void reportExceptions(ArrayList exceptions,
	        DataSourceQuery dataSourceQuery, PlugInContext context) {
	        context.getOutputFrame().addHeader(1,
	            exceptions.size() + " problem" + StringUtil.s(exceptions.size()) +
	            " loading " + dataSourceQuery.toString() + "." +
	            ((exceptions.size() > 10) ? " First and last five:" : ""));
	        context.getOutputFrame().addText("See View / Log for stack traces");
	        context.getOutputFrame().append("<ul>");

	        Collection exceptionsToReport = exceptions.size() <= 10 ? exceptions
	                                                                : CollectionUtil.concatenate(Arrays.asList(
	                    new Collection[] {
	                        exceptions.subList(0, 5),
	                        exceptions.subList(exceptions.size() - 5,
	                            exceptions.size())
	                    }));
	        for (Iterator j = exceptionsToReport.iterator(); j.hasNext();) {
	            Exception exception = (Exception) j.next();
	            context.getWorkbenchFrame().log(StringUtil.stackTrace(exception));
	            context.getOutputFrame().append("<li>");
	            context.getOutputFrame().append(GUIUtil.escapeHTML(
	                    WorkbenchFrame.toMessage(exception), true, true));
	            context.getOutputFrame().append("</li>");
	        }
	        context.getOutputFrame().append("</ul>");
	    }

 
}
