/*
 * Created on 22.05.2006 for PIROL
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2446 $
 *  $Date: 2006-09-12 14:57:25 +0200 (Di, 12 Sep 2006) $
 *  $Id: ChooseLayerAndAttributeOrDefaultValueBox.java 2446 2006-09-12 12:57:25Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.ui.panels;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextArea;

import com.vividsolutions.jump.feature.AttributeType;
import com.vividsolutions.jump.workbench.model.Layer;
import com.vividsolutions.jump.workbench.plugin.PlugInContext;

import de.fhOsnabrueck.jump.pirol.ui.documents.NumberInputDocument;
import de.fhOsnabrueck.jump.pirol.ui.documents.ObjectInputDocument;
import de.fhOsnabrueck.jump.pirol.ui.tools.DialogTools;
import de.fhOsnabrueck.jump.pirol.ui.tools.ValueChecker;
import de.fhOsnabrueck.jump.pirol.utilities.comparisonAndSorting.ObjectComparator;
import de.fhOsnabrueck.jump.pirol.utilities.i18n.PirolPlugInMessages;

/**
 * TODO: comment class
 *
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2006),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2446 $
 * 
 */
public class ChooseLayerAndAttributeOrDefaultValueBox extends Box implements ActionListener, ValueChecker {
    
    protected PlugInContext context = null;
    
    protected ChooseLayerAndAttributeBox layerAttributeBox = null;
    protected JCheckBox haveLayerCb = new JCheckBox();
    protected JTextArea defaultValueArea = new JTextArea();
    
    protected List allowedAttributes = null;
    
    protected String text = null;
    
    protected final static Dimension textAreaSize = new Dimension(35,20); 
    
    /**
     * 
     */
    private static final long serialVersionUID = -8566614228133184489L;

    public ChooseLayerAndAttributeOrDefaultValueBox(PlugInContext context, List allowedAttributes, String textResourceKey, int widthWanted, String defaultValueStr) {
        super(BoxLayout.Y_AXIS);
        this.context = context;
        this.text = PirolPlugInMessages.getString(textResourceKey);
        this.allowedAttributes = allowedAttributes;
        
        this.setupGui(widthWanted, defaultValueStr);
    }

    /**
     *
     */
    protected void setupGui(int widthWanted, String defaultValueStr) {
        this.layerAttributeBox = new ChooseLayerAndAttributeBox(this.context, allowedAttributes);
        
        Box hbox = Box.createHorizontalBox();
        
        hbox.add(DialogTools.getPanelWithLabels(this.text, widthWanted/10));
        hbox.add(Box.createGlue());
        
        this.add(hbox);
        
        hbox = Box.createHorizontalBox();
        hbox.add(new JLabel(PirolPlugInMessages.getString("have-layer") + ": "));
        hbox.add(Box.createGlue());
        this.haveLayerCb.addActionListener(this);
        hbox.add(this.haveLayerCb);
        hbox.add(Box.createGlue());
        hbox.add(Box.createHorizontalStrut(20));
        hbox.add(new JLabel(PirolPlugInMessages.getString("substitution-value") + ": "));
        hbox.add(Box.createGlue());
        
        this.defaultValueArea.setRows(1);
        this.defaultValueArea.setBorder(BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        this.defaultValueArea.setColumns(10);
        this.defaultValueArea.setMinimumSize(textAreaSize);
        this.defaultValueArea.setMaximumSize(textAreaSize);
        this.defaultValueArea.setPreferredSize(textAreaSize);
        if (allowedAttributes.contains(AttributeType.STRING)){
            this.defaultValueArea.setDocument(new ObjectInputDocument());
        } else {
            this.defaultValueArea.setDocument(new NumberInputDocument());
        }
        this.defaultValueArea.setText(defaultValueStr);
        
        hbox.add(this.defaultValueArea);
        
        
        this.add(hbox);
        
        this.add(this.layerAttributeBox);
        
        this.haveLayerCb.setSelected(false);
        this.layerAttributeBox.setEnabled(false);
        this.layerAttributeBox.setVisible(false);
    }

    
    
    public boolean haveLayer() {
        return haveLayerCb.isSelected();
    }

    /**
     *@param arg0
     */
    public void actionPerformed(ActionEvent arg0) {
        // have layer value changed
        if (this.haveLayer()){
//            this.defaultValueArea.setEnabled(false);
            this.layerAttributeBox.setEnabled(true);
            this.layerAttributeBox.setVisible(true);
        } else {
//            this.defaultValueArea.setEnabled(true);
            this.layerAttributeBox.setEnabled(false);
            this.layerAttributeBox.setVisible(false);
        }
    }

    public String getAttributeName() {
        return (this.haveLayer()?layerAttributeBox.getAttributeName():null);
    }

    public Layer getSelectedLayer() {
        return (this.haveLayer()?layerAttributeBox.getSelectedLayer():null);
    }

    public Object getDefaulValue() {
        if (defaultValueArea.getText()==null || defaultValueArea.getText().length()==0){
            return null;
        }
        
        if (this.allowedAttributes.contains(AttributeType.STRING)){
            return defaultValueArea.getText();
        }
        return new Double(ObjectComparator.getDoubleValue(this.defaultValueArea.getText()));
    }

    public boolean areValuesOk() {
        return this.getDefaulValue()!=null && ((this.haveLayer() && this.getSelectedLayer()!=null && this.getAttributeName()!=null) || !this.haveLayer());
    }

    public void setVisible(boolean visible) {
        super.setVisible(visible);
        this.layerAttributeBox.setVisible(this.haveLayer());
    }

    
}
