/*
 * Created on 10.04.2006 for PIROL
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2588 $
 *  $Date: 2006-10-24 10:36:51 +0200 (Di, 24 Okt 2006) $
 *  $Id: AbstractProjection.java 2588 2006-10-24 08:36:51Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.utilities.projection.Projections;

import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.DebugUserIds;
import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.PersonalLogger;
import de.fhOsnabrueck.jump.pirol.utilities.projection.ellipsoid.AbstractEllipsoid;

/**
 * Abstract base class for classes that descibe different projections
 *
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2006),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2588 $
 * 
 */
public abstract class AbstractProjection {
    
    public final static String KEY_GAUSS_KRUEGER = "Gau-Krger";
    public final static String KEY_WGS84 = "WGS84";

    protected final static PersonalLogger logger = new PersonalLogger(DebugUserIds.OLE);
    
    protected AbstractEllipsoid ellipsoid = null;
    
    
    public abstract int getSRID();
    
    abstract String asString(); 
    

    public AbstractEllipsoid getEllipsoid() {
        return ellipsoid;
    }

    protected void setEllipsoid(AbstractEllipsoid ellipsoid) {
        this.ellipsoid = ellipsoid;
    }

    public boolean equals(Object arg0) {
        return this.getClass().isInstance(arg0) && this.getSRID() == ((AbstractProjection)arg0).getSRID();
    }
    
    /**
     * 
     *@param srid
     *@return
     */
    public static AbstractProjection createInstance(int srid){
        if (Wgs84Projection.ESRID_WGS84 == srid){
            return new Wgs84Projection();
        } else if (srid >= GaussKruegerProjection.GAUSS_KRUEGER_BASE_SRID && srid < GaussKruegerProjection.GAUSS_KRUEGER_MAX_SRID) {
            return new GaussKruegerProjection(srid-GaussKruegerProjection.GAUSS_KRUEGER_BASE_SRID);
        }
        return null;
    }
}
